@@ -81,6 +81,7 @@ import (
 	"github.com/go-delve/delve/pkg/elfwriter"
 	"github.com/go-delve/delve/pkg/logflags"
 	"github.com/go-delve/delve/pkg/proc"
+	"github.com/go-delve/delve/pkg/proc/internal/ebpf"
 	"github.com/go-delve/delve/pkg/proc/linutil"
 	"github.com/go-delve/delve/pkg/proc/macutil"
 	isatty "github.com/mattn/go-isatty"
@@ -361,6 +362,18 @@ func (p *gdbProcess) Connect(conn net.Conn, path string, pid int, debugInfoDirs
 	return tgt, nil
 }
 
+func (p *gdbProcess) SupportsBPF() bool {
+	return false
+}
+
+func (dbp *gdbProcess) GetBufferedTracepoints() []ebpf.RawUProbeParams {
+	return nil
+}
+
+func (dbp *gdbProcess) SetUProbe(fnName string, args []ebpf.UProbeArgMap) error {
+	panic("not implemented")
+}
+
 // unusedPort returns an unused tcp port
 // This is a hack and subject to a race condition with other running
 // programs, but most (all?) OS will cycle through all ephemeral ports
@@ -1185,7 +1198,7 @@ func (p *gdbProcess) ChangeDirection(dir proc.Direction) error {
 	if p.conn.direction == dir {
 		return nil
 	}
-	if p.Breakpoints().HasInternalBreakpoints() {
+	if p.Breakpoints().HasSteppingBreakpoints() {
 		return ErrDirChange
 	}
 	p.conn.direction = dir
@@ -1793,7 +1806,7 @@ func (t *gdbThread) SetCurrentBreakpoint(adjustPC bool) error {
 				return err
 			}
 		}
-		t.CurrentBreakpoint = bp.CheckCondition(t)
+		t.CurrentBreakpoint.Breakpoint = bp
 	}
 	return nil
 }
