@@ -1,6 +1,9 @@
 package version
 
-import "fmt"
+import (
+	"fmt"
+	"runtime"
+)
 
 // Version represents the current version of Delve.
 type Version struct {
@@ -14,8 +17,8 @@ type Version struct {
 var (
 	// DelveVersion is the current version of Delve.
 	DelveVersion = Version{
-		Major: "1", Minor: "7", Patch: "0", Metadata: "",
-		Build: "$Id: e353a65161e6ed74952b96bbb62ebfc56090832b $",
+		Major: "1", Minor: "7", Patch: "1", Metadata: "",
+		Build: "$Id: 3bde2354aafb5a4043fd59838842c4cd4a8b6f0b $",
 	}
 )
 
@@ -26,3 +29,11 @@ func (v Version) String() string {
 	}
 	return fmt.Sprintf("%s\nBuild: %s", ver, v.Build)
 }
+
+var buildInfo = func() string {
+	return ""
+}
+
+func BuildInfo() string {
+	return fmt.Sprintf("%s\n%s", runtime.Version(), buildInfo())
+}
